@@ -3,14 +3,14 @@
 use super::bin::{BinInfo, BinLimits, BinRemapper};
 use super::empty_subgrid::EmptySubgridV1;
 use super::fk_table::FkTable;
-use super::import_only_subgrid::ImportOnlySubgridV1;
+use super::import_only_subgrid::ImportOnlySubgridV2;
 use super::lagrange_subgrid::{LagrangeSparseSubgridV1, LagrangeSubgridV1, LagrangeSubgridV2};
 use super::lumi::{LumiCache, LumiEntry};
 use super::lumi_entry;
 use super::ntuple_subgrid::NtupleSubgridV1;
 use super::pids;
 use super::sparse_array3::SparseArray3;
-use super::subgrid::{ExtraSubgridParams, Subgrid, SubgridEnum, SubgridParams};
+use super::subgrid::{ExtraSubgridParams, Mu2, Subgrid, SubgridEnum, SubgridParams};
 use float_cmp::approx_eq;
 use git_version::git_version;
 use indicatif::{ProgressBar, ProgressStyle};
@@ -1101,7 +1101,7 @@ impl Grid {
                         mem::swap(subgrid, &mut new_subgrid);
                     }
                     SubgridEnum::LagrangeSubgridV2(grid) => {
-                        let mut new_subgrid = ImportOnlySubgridV1::from(&*grid).into();
+                        let mut new_subgrid = ImportOnlySubgridV2::from(&*grid).into();
                         mem::swap(subgrid, &mut new_subgrid);
                     }
                     SubgridEnum::EmptySubgridV1(_)
@@ -1475,7 +1475,10 @@ impl Grid {
             .collect();
 
         // create target subgrid dimensions
-        let tgt_q2_grid = vec![eko_info.muf2_0];
+        let tgt_q2_grid = vec![Mu2 {
+            ren: eko_info.muf2_0,
+            fac: eko_info.muf2_0,
+        }];
         let tgt_x1_grid = if has_pdf1 {
             eko_info.target_x_grid.clone()
         } else {
@@ -1528,46 +1531,6 @@ impl Grid {
             })
             .collect();
 
-        // the x1 and x2 grid values are the same across all non-zero subgrids
-        let first_non_zero_grid = self
-            .subgrids
-            .iter()
-            .find(|subgrid| !subgrid.is_empty())
-            .unwrap_or_else(|| unreachable!());
-        // source x1/x2 grid might differ and be differently sorted than the operator
-        let x1_grid = if has_pdf1 {
-            first_non_zero_grid
-                .x1_grid()
-                .iter()
-                .map(|x| {
-                    eko_info
-                        .grid_axes
-                        .x_grid
-                        .iter()
-                        .position(|xi| approx_eq!(f64, *xi, *x, ulps = 64))
-                        .unwrap_or_else(|| unreachable!())
-                })
-                .collect()
-        } else {
-            Vec::new()
-        };
-        let x2_grid = if has_pdf2 {
-            first_non_zero_grid
-                .x2_grid()
-                .iter()
-                .map(|x| {
-                    eko_info
-                        .grid_axes
-                        .x_grid
-                        .iter()
-                        .position(|xi| approx_eq!(f64, *xi, *x, ulps = 64))
-                        .unwrap_or_else(|| unreachable!())
-                })
-                .collect()
-        } else {
-            Vec::new()
-        };
-
         // iterate over all bins, which are mapped one-to-one from the target to the source grid
         for bin in 0..self.bin_info().bins() {
             // iterate over the source grid luminosities
@@ -1616,6 +1579,40 @@ impl Grid {
 
                     let src_subgrid = &self.subgrids[[order, bin, src_lumi]];
 
+                    // source x1/x2 grid might differ and be differently sorted than the operator
+                    let x1_grid = if has_pdf1 {
+                        src_subgrid
+                            .x1_grid()
+                            .iter()
+                            .map(|x| {
+                                eko_info
+                                    .grid_axes
+                                    .x_grid
+                                    .iter()
+                                    .position(|xi| approx_eq!(f64, *xi, *x, ulps = 64))
+                                    .unwrap_or_else(|| unreachable!())
+                            })
+                            .collect()
+                    } else {
+                        Vec::new()
+                    };
+                    let x2_grid = if has_pdf2 {
+                        src_subgrid
+                            .x2_grid()
+                            .iter()
+                            .map(|x| {
+                                eko_info
+                                    .grid_axes
+                                    .x_grid
+                                    .iter()
+                                    .position(|xi| approx_eq!(f64, *xi, *x, ulps = 64))
+                                    .unwrap_or_else(|| unreachable!())
+                            })
+                            .collect()
+                    } else {
+                        Vec::new()
+                    };
+
                     for ((iq2, ix1, ix2), &value) in src_subgrid.iter() {
                         let scale = src_subgrid.mu2_grid()[iq2].fac;
                         let src_iq2 = src_array_q2_grid
@@ -1782,14 +1779,14 @@ impl Grid {
                             );
 
                             let mut subgrid = match tgt_subgrid {
-                                SubgridEnum::EmptySubgridV1(_) => ImportOnlySubgridV1::new(
+                                SubgridEnum::EmptySubgridV1(_) => ImportOnlySubgridV2::new(
                                     tgt_array,
                                     tgt_q2_grid.clone(),
                                     tgt_x1_grid.clone(),
                                     tgt_x2_grid.clone(),
                                 )
                                 .into(),
-                                SubgridEnum::ImportOnlySubgridV1(ref mut array) => {
+                                SubgridEnum::ImportOnlySubgridV2(ref mut array) => {
                                     let array = array.array_mut();
 
                                     for ((_, tgt_x1_idx, tgt_x2_idx), &value) in
@@ -1933,6 +1930,7 @@ impl Grid {
 #[cfg(test)]
 mod tests {
     use super::*;
+    use crate::import_only_subgrid::ImportOnlySubgridV1;
     use crate::lumi_entry;
 
     #[test]
