@@ -180,11 +180,11 @@ impl PyGrid {
         weights: PyReadonlyArray1<f64>,
     ) {
         for (&x1, &x2, &q2, &observable, &weight) in izip!(
-            x1s.iter().unwrap(),
-            x2s.iter().unwrap(),
-            q2s.iter().unwrap(),
-            observables.iter().unwrap(),
-            weights.iter().unwrap(),
+            x1s.as_array().iter(),
+            x2s.as_array().iter(),
+            q2s.as_array().iter(),
+            observables.as_array().iter(),
+            weights.as_array().iter(),
         ) {
             self.grid.fill(
                 order,
@@ -297,20 +297,29 @@ impl PyGrid {
     ///         interpolation grid
     ///     pids: numpy.ndarray(int)
     ///         particle ids
+    ///     mur2_grid : numpy.ndarray(float)
+    ///         factorization scale list
     ///     muf2_grid : numpy.ndarray(float)
     ///         factorization scale list
     pub fn axes<'py>(
         &self,
         py: Python<'py>,
-    ) -> (&'py PyArray1<f64>, &'py PyArray1<i32>, &'py PyArray1<f64>) {
+    ) -> (
+        &'py PyArray1<f64>,
+        &'py PyArray1<i32>,
+        &'py PyArray1<f64>,
+        &'py PyArray1<f64>,
+    ) {
         let GridAxes {
             x_grid,
             pids,
+            mur2_grid,
             muf2_grid,
         } = self.grid.axes().unwrap();
         (
             x_grid.into_pyarray(py),
             pids.into_pyarray(py),
+            mur2_grid.into_pyarray(py),
             muf2_grid.into_pyarray(py),
         )
     }
@@ -392,8 +401,10 @@ impl PyGrid {
     ///         sorting of the particles in the tensor for final FkTable
     ///     target_x_grid : numpy.ndarray(float)
     ///         final FKTable interpolation grid
+    ///     mur2_grid : numpy.ndarray(float)
+    ///         list of renormalization scales
     ///     muf2_grid : numpy.ndarray(float)
-    ///         list of process scales
+    ///         list of factorization scales
     ///     operator : numpy.ndarray(int, rank=5)
     ///         evolution tensor
     ///     orders_mask: numpy.ndarray(bool)
@@ -411,21 +422,24 @@ impl PyGrid {
         x_grid: PyReadonlyArray1<f64>,
         target_pids: PyReadonlyArray1<i32>,
         target_x_grid: PyReadonlyArray1<f64>,
+        mur2_grid: PyReadonlyArray1<f64>,
         muf2_grid: PyReadonlyArray1<f64>,
         operator: PyReadonlyArray5<f64>,
         lumi_id_types: String,
         order_mask: PyReadonlyArray1<bool>,
+        xi: (f64, f64),
     ) -> PyFkTable {
         let eko_info = EkoInfo {
             muf2_0,
             alphas: alphas.to_vec().unwrap(),
-            xir: 1.,
-            xif: 1.,
+            xir: xi.0,
+            xif: xi.1,
             target_pids: target_pids.to_vec().unwrap(),
             target_x_grid: target_x_grid.to_vec().unwrap(),
             grid_axes: GridAxes {
                 x_grid: x_grid.to_vec().unwrap(),
                 pids: pids.to_vec().unwrap(),
+                mur2_grid: mur2_grid.to_vec().unwrap(),
                 muf2_grid: muf2_grid.to_vec().unwrap(),
             },
             lumi_id_types,
@@ -438,7 +452,7 @@ impl PyGrid {
                 eko_info,
                 &order_mask.to_vec().unwrap(),
             )
-            .unwrap();
+            .expect("Nothing returned from evolution.");
         PyFkTable {
             fk_table: evolved_grid,
         }
@@ -580,6 +594,21 @@ impl PyGrid {
             .collect()
     }
 
+    /// Get luminsosity functions.
+    ///
+    /// Returns
+    /// -------
+    ///     list(list(tuple(float,float))) :
+    ///         luminosity functions as pid tuples (multiple tuples can bee associated to the same
+    ///         contribution)
+    pub fn lumi(&self) -> Vec<Vec<(i32, i32, f64)>> {
+        self.grid
+            .lumi()
+            .iter()
+            .map(|entry| entry.entry().to_vec())
+            .collect()
+    }
+
     /// Scale all subgrids.
     ///
     /// Parameters
@@ -590,13 +619,23 @@ impl PyGrid {
         self.grid.scale(factor);
     }
 
+    /// Scale subgrids bin by bin.
+    ///
+    /// Parameters
+    /// ----------
+    /// factors : numpy.ndarray[float]
+    ///     bin-dependent factors by which scaling
+    pub fn scale_by_bin(&mut self, factors: PyReadonlyArray1<f64>) {
+        self.grid.scale_by_bin(&factors.to_vec().unwrap());
+    }
+
     /// Delete bins.
     ///
     /// Repeated bins and those exceeding length are ignored.
     ///
     /// Parameters
     /// ----------
-    /// bin_indices : numpy.ndarray(int)
+    /// bin_indices : numpy.ndarray[int]
     ///     list of indices of bins to removed
     pub fn delete_bins(&mut self, bin_indices: PyReadonlyArray1<usize>) {
         self.grid.delete_bins(&bin_indices.to_vec().unwrap())
